@@ -16,7 +16,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-#ident "$Id: keymap.c,v 1.11 2003/01/14 03:04:30 nalin Exp $"
+#ident "$Id: keymap.c,v 1.14 2003/01/17 20:24:57 nalin Exp $"
 #include "../config.h"
 #include <stdlib.h>
 #include <stdio.h>
@@ -857,7 +857,7 @@ _vte_keymap_map(guint keyval,
 	enum _vte_fkey_mode fkey_mode;
 	char *cap, *tmp;
 	const char *termcap_special = NULL;
-#ifdef HAVE_NCURSES
+#if defined(HAVE_NCURSES) || defined(HAVE_TERMCAP)
 	char ncurses_buffer[4096];
 	char ncurses_area[512];
 #endif
@@ -1061,43 +1061,49 @@ _vte_keymap_map(guint keyval,
 #if defined(HAVE_NCURSES) || defined(HAVE_TERMCAP)
 	if (termcap_special != NULL) {
 		tmp = g_strdup(terminal);
+		cap = NULL;
 		if (tgetent(ncurses_buffer, tmp) == 1) {
 			cap = ncurses_area;
-			g_free(tmp);
 			tmp = g_strdup(termcap_special);
 			cap = tgetstr(tmp, &cap);
-			if ((cap != NULL) && (*cap != '\0')) {
-				g_free(tmp);
-				*normal_length = strlen(cap);
-				*normal = g_strdup(cap);
-#ifdef VTE_DEBUG
-				if (_vte_debug_on(VTE_DEBUG_KEYBOARD)) {
-					int j;
-#ifdef HAVE_NCURSES
-					fprintf(stderr, " via ncurses to '");
-#else
-#ifdef HAVE_TERMCAP
-					fprintf(stderr, " via termcap to '");
-#endif
-#endif
-					for (j = 0; j < *normal_length; j++) {
-						if (((*normal)[j] < 32) ||
-						    ((*normal)[j] >= 127)) {
-							fprintf(stderr, "<0x%02x>",
-								(*normal)[j]);
-						} else {
-							fprintf(stderr, "%c",
-								(*normal)[j]);
-						}
-					}
-					fprintf(stderr,
-						"', returning.\n");
-				}
-#endif
-				return;
+		}
+		if ((cap == NULL) && (strcmp(terminal, "xterm") == 0)) {
+			/* try, try again */
+			if (tgetent(ncurses_buffer, "xterm-xfree86") == 1) {
+				cap = ncurses_area;
+				tmp = g_strdup(termcap_special);
+				cap = tgetstr(tmp, &cap);
 			}
 		}
 		g_free(tmp);
+		if ((cap != NULL) && (*cap != '\0')) {
+			*normal_length = strlen(cap);
+			*normal = g_strdup(cap);
+#ifdef VTE_DEBUG
+			if (_vte_debug_on(VTE_DEBUG_KEYBOARD)) {
+				int j;
+#ifdef HAVE_NCURSES
+				fprintf(stderr, " via ncurses to '");
+#else
+#ifdef HAVE_TERMCAP
+				fprintf(stderr, " via termcap to '");
+#endif
+#endif
+				for (j = 0; j < *normal_length; j++) {
+					if (((*normal)[j] < 32) ||
+					    ((*normal)[j] >= 127)) {
+						fprintf(stderr, "<0x%02x>",
+							(*normal)[j]);
+					} else {
+						fprintf(stderr, "%c",
+							(*normal)[j]);
+					}
+				}
+				fprintf(stderr, "', returning.\n");
+			}
+#endif
+			return;
+		}
 	}
 #endif
 
@@ -1277,7 +1283,7 @@ _vte_keymap_key_add_key_modifiers(guint keyval,
 		return;
 	}
 
-	nnormal = g_malloc0(*normal_length + 3);
+	nnormal = g_malloc0(*normal_length + 4);
 	memcpy(nnormal, *normal, *normal_length);
 	if (strlen(nnormal) > 1) {
 		offset = strlen(nnormal) - 1;
@@ -1287,9 +1293,11 @@ _vte_keymap_key_add_key_modifiers(guint keyval,
 			nnormal[offset + 0] = ';';
 			*normal_length += 2;
 		} else {
-			nnormal[offset + 1] = nnormal[offset];
-			nnormal[offset + 0] = modifier + '0';
-			*normal_length += 1;
+			nnormal[offset + 3] = nnormal[offset];
+			nnormal[offset + 2] = modifier + '0';
+			nnormal[offset + 1] = ';';
+			nnormal[offset + 0] = '1';
+			*normal_length += 3;
 		}
 		g_free(*normal);
 		*normal = nnormal;
