@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2002 Red Hat, Inc.
+ * Copyright (C) 2002,2003 Red Hat, Inc.
  *
  * This is free software; you can redistribute it and/or modify it under
  * the terms of the GNU Library General Public License as published by
@@ -19,10 +19,11 @@
 /* VTE accessibility object.  Based heavily on inspection of libzvt's
  * accessibility code. */
 
-#ident "$Id: vteaccess.c,v 1.21 2002/12/18 20:02:42 nalin Exp $"
+#ident "$Id: vteaccess.c,v 1.23 2003/01/14 03:04:30 nalin Exp $"
 #include "../config.h"
 #include <atk/atk.h>
 #include <gtk/gtk.h>
+#include <string.h>
 #include "debug.h"
 #include "vte.h"
 #include "vteaccess.h"
@@ -74,13 +75,90 @@ vte_terminal_accessible_new_private_data(void)
 
 /* "Oh yeah, that's selected.  Sure."  */
 static gboolean
-all_selected(VteTerminal *terminal, glong column, glong row)
+all_selected(VteTerminal *terminal, glong column, glong row, gpointer data)
 {
 	return TRUE;
 }
 
 static void
-vte_terminal_accessible_update_private_data_if_needed(AtkObject *text)
+emit_text_caret_moved(GObject *object, glong caret)
+{
+#ifdef VTE_DEBUG
+	if (_vte_debug_on(VTE_DEBUG_SIGNALS)) {
+		fprintf(stderr, "Accessibility peer emitting "
+			"`text-caret-moved'.\n");
+	}
+#endif
+	g_signal_emit_by_name(object, "text-caret-moved", caret);
+}
+
+static void
+emit_text_changed_insert(GObject *object,
+			 const char *text, glong offset, glong len)
+{
+	const char *p;
+	glong start, count;
+	if (len == 0) {
+		return;
+	}
+	/* Convert the byte offsets to characters. */
+	for (p = text, start = 0;
+	     p < text + offset;
+	     p = g_utf8_next_char(p)) {
+		start++;
+	}
+	for (p = text + offset, count = 0;
+	     p < text + offset + len;
+	     p = g_utf8_next_char(p)) {
+		count++;
+	}
+#ifdef VTE_DEBUG
+	if (_vte_debug_on(VTE_DEBUG_SIGNALS)) {
+		fprintf(stderr, "Accessibility peer emitting "
+			"`text-changed::insert' (%ld, %ld) (%ld, %ld).\n",
+			offset, len, start, count);
+		fprintf(stderr, "Inserted text was `%.*s'.\n",
+			(int) len, text + offset);
+	}
+#endif
+	g_signal_emit_by_name(object, "text-changed::insert", start, count);
+}
+
+static void
+emit_text_changed_delete(GObject *object,
+			 const char *text, glong offset, glong len)
+{
+	const char *p;
+	glong start, count;
+	if (len == 0) {
+		return;
+	}
+	/* Convert the byte offsets to characters. */
+	for (p = text, start = 0;
+	     p < text + offset;
+	     p = g_utf8_next_char(p)) {
+		start++;
+	}
+	for (p = text + offset, count = 0;
+	     p < text + offset + len;
+	     p = g_utf8_next_char(p)) {
+		count++;
+	}
+#ifdef VTE_DEBUG
+	if (_vte_debug_on(VTE_DEBUG_SIGNALS)) {
+		fprintf(stderr, "Accessibility peer emitting "
+			"`text-changed::delete' (%ld, %ld) (%ld, %ld).\n",
+			offset, len, start, count);
+		fprintf(stderr, "Deleted text was `%.*s'.\n",
+			(int) len, text + offset);
+	}
+#endif
+	g_signal_emit_by_name(object, "text-changed::delete", start, count);
+}
+
+static void
+vte_terminal_accessible_update_private_data_if_needed(AtkObject *text,
+						      char **old)
 {
 	VteTerminal *terminal;
 	VteTerminalAccessiblePrivate *priv;
@@ -99,6 +177,13 @@ vte_terminal_accessible_update_private_data_if_needed(AtkObject *text)
 	/* If nothing's changed, just return immediately. */
 	if ((priv->snapshot_contents_invalid == FALSE) &&
 	    (priv->snapshot_caret_invalid == FALSE)) {
+		if (old) {
+			if (priv->snapshot_text) {
+				*old = g_strdup(priv->snapshot_text);
+			} else {
+				*old = g_strdup("");
+			}
+		}
 		return;
 	}
 
@@ -106,11 +191,18 @@ vte_terminal_accessible_update_private_data_if_needed(AtkObject *text)
 
 	/* Re-read the contents of the widget if the contents have changed. */
 	if (priv->snapshot_contents_invalid) {
-		/* Free the outdated snapshot data. */
-		if (priv->snapshot_text != NULL) {
+		/* Free the outdated snapshot data, unless the caller
+		 * wants it. */
+		if (old) {
+			if (priv->snapshot_text != NULL) {
+				*old = priv->snapshot_text;
+			} else {
+				*old = g_strdup("");
+			}
+		} else {
 			g_free(priv->snapshot_text);
-			priv->snapshot_text = NULL;
 		}
+		priv->snapshot_text = NULL;
 
 		/* Free the character offsets and allocate a new array to hold
 		 * them. */
@@ -219,8 +311,7 @@ vte_terminal_accessible_update_private_data_if_needed(AtkObject *text)
 		/* The caret may have moved. */
 		if (caret != priv->snapshot_caret) {
 			priv->snapshot_caret = caret;
-			g_signal_emit_by_name(G_OBJECT(text), "text_caret_moved",
-					      caret);
+			emit_text_caret_moved(G_OBJECT(text), caret);
 		}
 		/* Done updating the caret position, too. */
 		priv->snapshot_caret_invalid = FALSE;
@@ -257,12 +348,13 @@ vte_terminal_accessible_free_private_data(VteTerminalAccessiblePrivate *priv)
 	g_free(priv);
 }
 
-/* A signal handler to catch "contents_changed" signals. */
+/* A signal handler to catch "text-inserted/deleted/modified" signals. */
 static void
-vte_terminal_accessible_invalidate_contents(VteTerminal *terminal,
-					    gpointer data)
+vte_terminal_accessible_text_modified(VteTerminal *terminal, gpointer data)
 {
 	VteTerminalAccessiblePrivate *priv;
+	char *old;
+	glong offset, olen, nlen;
 
 	g_return_if_fail(VTE_IS_TERMINAL_ACCESSIBLE(data));
 
@@ -270,13 +362,179 @@ vte_terminal_accessible_invalidate_contents(VteTerminal *terminal,
 				 VTE_TERMINAL_ACCESSIBLE_PRIVATE_DATA);
 	g_return_if_fail(priv != NULL);
 
-#ifdef VTE_DEBUG
-	if (_vte_debug_on(VTE_DEBUG_MISC)) {
-		fprintf(stderr, "Invalidating accessibility contents.\n");
-	}
-#endif
-
 	priv->snapshot_contents_invalid = TRUE;
+	vte_terminal_accessible_update_private_data_if_needed(ATK_OBJECT(data),
+							      &old);
+	g_return_if_fail(old != NULL);
+
+	olen = strlen(old);
+	nlen = strlen(priv->snapshot_text);
+
+	/* Find the offset where they don't match. */
+	offset = 0;
+	while ((offset < olen) && (offset < nlen)) {
+		if (old[offset] != priv->snapshot_text[offset]) {
+			break;
+		}
+		offset++;
+	}
+
+	/* At least one of them had better have more data, right? */
+	if ((offset < olen) || (offset < nlen)) {
+		/* Back up both end points until we find the *last* point
+		 * where they differed. */
+		while ((olen > offset) && (nlen > offset)) {
+			if (old[olen - 1] !=
+			    priv->snapshot_text[nlen - 1]) {
+				break;
+			}
+			olen--;
+			nlen--;
+		}
+		/* At least one of them has to have text the other
+		 * doesn't. */
+		g_assert((nlen > offset) || (olen > offset));
+		/* Now emit a deleted signal for text that was in the old
+		 * string but isn't in the new one... */
+		emit_text_changed_delete(G_OBJECT(data),
+					 old,
+					 offset, olen - offset);
+		/* .. and an inserted signal for text that wasn't in the old
+		 * string but is in the new one. */
+		emit_text_changed_insert(G_OBJECT(data),
+					 priv->snapshot_text,
+					 offset, nlen - offset);
+	}
+
+	g_free(old);
+}
+
+/* A signal handler to catch "text-scrolled" signals. */
+static void
+vte_terminal_accessible_text_scrolled(VteTerminal *terminal,
+				      gint howmuch,
+				      gpointer data)
+{
+	VteTerminalAccessiblePrivate *priv;
+	struct vte_char_attributes attr;
+	long i, len, delta;
+
+	g_return_if_fail(VTE_IS_TERMINAL_ACCESSIBLE(data));
+	g_return_if_fail(howmuch != 0);
+
+	priv = g_object_get_data(G_OBJECT(data),
+				 VTE_TERMINAL_ACCESSIBLE_PRIVATE_DATA);
+	g_return_if_fail(priv != NULL);
+
+	if (((howmuch < 0) && (howmuch <= -terminal->row_count)) ||
+	    ((howmuch > 0) && (howmuch >= terminal->row_count))) {
+		/* All of the text was removed. */
+		if (priv->snapshot_text != NULL) {
+			if (priv->snapshot_text) {
+				emit_text_changed_delete(G_OBJECT(data),
+							 priv->snapshot_text,
+							 0,
+							 strlen(priv->snapshot_text));
+			}
+		}
+		priv->snapshot_contents_invalid = TRUE;
+		vte_terminal_accessible_update_private_data_if_needed(ATK_OBJECT(data),
+								      NULL);
+		/* All of the present text was added. */
+		if (priv->snapshot_text != NULL) {
+			if (priv->snapshot_text) {
+				emit_text_changed_insert(G_OBJECT(data),
+							 priv->snapshot_text,
+							 0,
+							 strlen(priv->snapshot_text));
+			}
+		}
+		return;
+	}
+	/* Find the start point. */
+	delta = 0;
+	if (priv->snapshot_attributes != NULL) {
+		if (priv->snapshot_attributes->len > 0) {
+			attr = g_array_index(priv->snapshot_attributes,
+					     struct vte_char_attributes,
+					     0);
+			delta = attr.row;
+		}
+	}
+	/* We scrolled up, so text was added at the top and removed
+	 * from the bottom. */
+	if ((howmuch < 0) && (howmuch > -terminal->row_count)) {
+		howmuch = -howmuch;
+		/* Find the first byte that scrolled off. */
+		for (i = 0; i < priv->snapshot_attributes->len; i++) {
+			attr = g_array_index(priv->snapshot_attributes,
+					     struct vte_char_attributes,
+					     i);
+			if (attr.row >= delta + terminal->row_count - howmuch) {
+				break;
+			}
+		}
+		if (i < priv->snapshot_attributes->len) {
+			/* The rest of the string was deleted -- make a note. */
+			emit_text_changed_delete(G_OBJECT(data),
+						 priv->snapshot_text,
+						 i,
+						 priv->snapshot_attributes->len - i);
+		}
+		/* Refresh.  Note that i is now the length of the data which
+		 * we expect to have left over. */
+		priv->snapshot_contents_invalid = TRUE;
+		vte_terminal_accessible_update_private_data_if_needed(ATK_OBJECT(data),
+								      NULL);
+		/* If we now have more text than before, the initial portion
+		 * was added. */
+		if (priv->snapshot_text != NULL) {
+			len = strlen(priv->snapshot_text);
+			if (len > i) {
+				emit_text_changed_insert(G_OBJECT(data),
+							 priv->snapshot_text,
+							 0,
+							 len - i);
+			}
+		}
+		return;
+	}
+	/* We scrolled down, so text was added at the bottom and removed
+	 * from the top. */
+	if ((howmuch > 0) && (howmuch < terminal->row_count)) {
+		/* Find the first byte that wasn't scrolled off the top. */
+		for (i = 0; i < priv->snapshot_attributes->len; i++) {
+			attr = g_array_index(priv->snapshot_attributes,
+					     struct vte_char_attributes,
+					     i);
+			if (attr.row >= delta + howmuch) {
+				break;
+			}
+		}
+		/* That many bytes disappeared -- make a note. */
+		emit_text_changed_delete(G_OBJECT(data),
+					 priv->snapshot_text,
+					 0,
+					 i);
+		/* Figure out how much text was left, and refresh. */
+		i = strlen(priv->snapshot_text + i);
+		priv->snapshot_contents_invalid = TRUE;
+		vte_terminal_accessible_update_private_data_if_needed(ATK_OBJECT(data),
+								      NULL);
+		/* Any newly-added string data is new, so note that it was
+		 * inserted. */
+		if (priv->snapshot_text != NULL) {
+			len = strlen(priv->snapshot_text);
+			if (len > i) {
+				emit_text_changed_insert(G_OBJECT(data),
+							 priv->snapshot_text,
+							 i,
+							 len - i);
+			}
+		}
+		return;
+	}
+	g_assert_not_reached();
 }
 
 /* A signal handler to catch "cursor-moved" signals. */
@@ -297,7 +555,8 @@ vte_terminal_accessible_invalidate_cursor(VteTerminal *terminal, gpointer data)
 	}
 #endif
 	priv->snapshot_caret_invalid = TRUE;
-	vte_terminal_accessible_update_private_data_if_needed(ATK_OBJECT(data));
+	vte_terminal_accessible_update_private_data_if_needed(ATK_OBJECT(data),
+							      NULL);
 }
 
 /* Handle title changes by resetting the parent object. */
@@ -337,8 +596,18 @@ vte_terminal_accessible_new(VteTerminal *terminal)
 	g_object_set_data(G_OBJECT(access),
 			  VTE_TERMINAL_ACCESSIBLE_PRIVATE_DATA,
 			  vte_terminal_accessible_new_private_data());
-	g_signal_connect(G_OBJECT(terminal), "contents-changed",
-			 GTK_SIGNAL_FUNC(vte_terminal_accessible_invalidate_contents),
+
+	g_signal_connect(G_OBJECT(terminal), "text-inserted",
+			 GTK_SIGNAL_FUNC(vte_terminal_accessible_text_modified),
+			 object);
+	g_signal_connect(G_OBJECT(terminal), "text-deleted",
+			 GTK_SIGNAL_FUNC(vte_terminal_accessible_text_modified),
+			 object);
+	g_signal_connect(G_OBJECT(terminal), "text-modified",
+			 GTK_SIGNAL_FUNC(vte_terminal_accessible_text_modified),
+			 object);
+	g_signal_connect(G_OBJECT(terminal), "text-scrolled",
+			 GTK_SIGNAL_FUNC(vte_terminal_accessible_text_scrolled),
 			 object);
 	g_signal_connect(G_OBJECT(terminal), "cursor-moved",
 			 GTK_SIGNAL_FUNC(vte_terminal_accessible_invalidate_cursor),
@@ -353,7 +622,10 @@ vte_terminal_accessible_new(VteTerminal *terminal)
 	}
 
 	atk_object_set_name(ATK_OBJECT(access), "Terminal");
-	atk_object_set_description(ATK_OBJECT(access), terminal->window_title ? terminal->window_title : "");
+	atk_object_set_description(ATK_OBJECT(access),
+				   terminal->window_title ?
+				   terminal->window_title :
+				   "");
 	
 	return ATK_OBJECT(access);
 }
@@ -372,7 +644,19 @@ vte_terminal_accessible_finalize(GObject *object)
 					     G_SIGNAL_MATCH_FUNC |
 					     G_SIGNAL_MATCH_DATA,
 					     0, 0, NULL,
-					     (gpointer)vte_terminal_accessible_invalidate_contents,
+					     vte_terminal_accessible_text_modified,
+					     object);
+	g_signal_handlers_disconnect_matched(G_OBJECT(accessible->widget),
+					     G_SIGNAL_MATCH_FUNC |
+					     G_SIGNAL_MATCH_DATA,
+					     0, 0, NULL,
+					     vte_terminal_accessible_text_scrolled,
+					     object);
+	g_signal_handlers_disconnect_matched(G_OBJECT(accessible->widget),
+					     G_SIGNAL_MATCH_FUNC |
+					     G_SIGNAL_MATCH_DATA,
+					     0, 0, NULL,
+					     vte_terminal_accessible_invalidate_cursor,
 					     object);
 	g_signal_handlers_disconnect_matched(G_OBJECT(accessible->widget),
 					     G_SIGNAL_MATCH_FUNC |
@@ -395,7 +679,8 @@ vte_terminal_accessible_get_text(AtkText *text,
 	g_return_val_if_fail((start_offset >= 0) && (end_offset >= -1),
 			     g_strdup(""));
 
-	vte_terminal_accessible_update_private_data_if_needed(ATK_OBJECT(text));
+	vte_terminal_accessible_update_private_data_if_needed(ATK_OBJECT(text),
+							      NULL);
 
 	priv = g_object_get_data(G_OBJECT(text),
 				 VTE_TERMINAL_ACCESSIBLE_PRIVATE_DATA);
@@ -443,7 +728,8 @@ vte_terminal_accessible_get_text_somewhere(AtkText *text,
 	gunichar current, prev, next;
 	int line;
 
-	vte_terminal_accessible_update_private_data_if_needed(ATK_OBJECT(text));
+	vte_terminal_accessible_update_private_data_if_needed(ATK_OBJECT(text),
+							      NULL);
 
 	priv = g_object_get_data(G_OBJECT(text),
 				 VTE_TERMINAL_ACCESSIBLE_PRIVATE_DATA);
@@ -610,7 +896,8 @@ vte_terminal_accessible_get_text_before_offset(AtkText *text, gint offset,
 					       gint *end_offset)
 {
 	g_return_val_if_fail(VTE_IS_TERMINAL_ACCESSIBLE(text), NULL);
-	vte_terminal_accessible_update_private_data_if_needed(ATK_OBJECT(text));
+	vte_terminal_accessible_update_private_data_if_needed(ATK_OBJECT(text),
+							      NULL);
 	return vte_terminal_accessible_get_text_somewhere(text,
 							  offset,
 							  boundary_type,
@@ -626,7 +913,8 @@ vte_terminal_accessible_get_text_after_offset(AtkText *text, gint offset,
 					      gint *end_offset)
 {
 	g_return_val_if_fail(VTE_IS_TERMINAL_ACCESSIBLE(text), NULL);
-	vte_terminal_accessible_update_private_data_if_needed(ATK_OBJECT(text));
+	vte_terminal_accessible_update_private_data_if_needed(ATK_OBJECT(text),
+							      NULL);
 	return vte_terminal_accessible_get_text_somewhere(text,
 							  offset,
 							  boundary_type,
@@ -642,7 +930,8 @@ vte_terminal_accessible_get_text_at_offset(AtkText *text, gint offset,
 					   gint *end_offset)
 {
 	g_return_val_if_fail(VTE_IS_TERMINAL_ACCESSIBLE(text), NULL);
-	vte_terminal_accessible_update_private_data_if_needed(ATK_OBJECT(text));
+	vte_terminal_accessible_update_private_data_if_needed(ATK_OBJECT(text),
+							      NULL);
 	return vte_terminal_accessible_get_text_somewhere(text,
 							  offset,
 							  boundary_type,
@@ -659,7 +948,8 @@ vte_terminal_accessible_get_character_at_offset(AtkText *text, gint offset)
 	char *unichar;
 	gunichar ret;
 
-	vte_terminal_accessible_update_private_data_if_needed(ATK_OBJECT(text));
+	vte_terminal_accessible_update_private_data_if_needed(ATK_OBJECT(text),
+							      NULL);
 
 	priv = g_object_get_data(G_OBJECT(text),
 				 VTE_TERMINAL_ACCESSIBLE_PRIVATE_DATA);
@@ -680,7 +970,8 @@ vte_terminal_accessible_get_caret_offset(AtkText *text)
 {
 	VteTerminalAccessiblePrivate *priv;
 
-	vte_terminal_accessible_update_private_data_if_needed(ATK_OBJECT(text));
+	vte_terminal_accessible_update_private_data_if_needed(ATK_OBJECT(text),
+							      NULL);
 
 	priv = g_object_get_data(G_OBJECT(text),
 				 VTE_TERMINAL_ACCESSIBLE_PRIVATE_DATA);
@@ -693,7 +984,8 @@ vte_terminal_accessible_get_run_attributes(AtkText *text, gint offset,
 					   gint *start_offset, gint *end_offset)
 {
 	g_return_val_if_fail(VTE_IS_TERMINAL_ACCESSIBLE(text), NULL);
-	vte_terminal_accessible_update_private_data_if_needed(ATK_OBJECT(text));
+	vte_terminal_accessible_update_private_data_if_needed(ATK_OBJECT(text),
+							      NULL);
 	/* FIXME */
 	return NULL;
 }
@@ -702,7 +994,8 @@ static AtkAttributeSet *
 vte_terminal_accessible_get_default_attributes(AtkText *text)
 {
 	g_return_val_if_fail(VTE_IS_TERMINAL_ACCESSIBLE(text), NULL);
-	vte_terminal_accessible_update_private_data_if_needed(ATK_OBJECT(text));
+	vte_terminal_accessible_update_private_data_if_needed(ATK_OBJECT(text),
+							      NULL);
 	/* FIXME */
 	return NULL;
 }
@@ -714,7 +1007,8 @@ vte_terminal_accessible_get_character_extents(AtkText *text, gint offset,
 					      AtkCoordType coords)
 {
 	g_return_if_fail(VTE_IS_TERMINAL_ACCESSIBLE(text));
-	vte_terminal_accessible_update_private_data_if_needed(ATK_OBJECT(text));
+	vte_terminal_accessible_update_private_data_if_needed(ATK_OBJECT(text),
+							      NULL);
 	/* FIXME */
 }
 
@@ -723,7 +1017,8 @@ vte_terminal_accessible_get_character_count(AtkText *text)
 {
 	VteTerminalAccessiblePrivate *priv;
 
-	vte_terminal_accessible_update_private_data_if_needed(ATK_OBJECT(text));
+	vte_terminal_accessible_update_private_data_if_needed(ATK_OBJECT(text),
+							      NULL);
 
 	priv = g_object_get_data(G_OBJECT(text),
 				 VTE_TERMINAL_ACCESSIBLE_PRIVATE_DATA);
@@ -737,7 +1032,8 @@ vte_terminal_accessible_get_offset_at_point(AtkText *text,
 					    AtkCoordType coords)
 {
 	g_return_val_if_fail(VTE_IS_TERMINAL_ACCESSIBLE(text), 0);
-	vte_terminal_accessible_update_private_data_if_needed(ATK_OBJECT(text));
+	vte_terminal_accessible_update_private_data_if_needed(ATK_OBJECT(text),
+							      NULL);
 	/* FIXME */
 	return 0;
 }
@@ -746,7 +1042,8 @@ static gint
 vte_terminal_accessible_get_n_selections(AtkText *text)
 {
 	g_return_val_if_fail(VTE_IS_TERMINAL_ACCESSIBLE(text), 0);
-	vte_terminal_accessible_update_private_data_if_needed(ATK_OBJECT(text));
+	vte_terminal_accessible_update_private_data_if_needed(ATK_OBJECT(text),
+							      NULL);
 	/* FIXME? */
 	return 0;
 }
@@ -756,7 +1053,8 @@ vte_terminal_accessible_get_selection(AtkText *text, gint selection_number,
 				      gint *start_offset, gint *end_offset)
 {
 	g_return_val_if_fail(VTE_IS_TERMINAL_ACCESSIBLE(text), NULL);
-	vte_terminal_accessible_update_private_data_if_needed(ATK_OBJECT(text));
+	vte_terminal_accessible_update_private_data_if_needed(ATK_OBJECT(text),
+							      NULL);
 	/* FIXME? */
 	return NULL;
 }
@@ -766,7 +1064,8 @@ vte_terminal_accessible_add_selection(AtkText *text,
 				      gint start_offset, gint end_offset)
 {
 	g_return_val_if_fail(VTE_IS_TERMINAL_ACCESSIBLE(text), FALSE);
-	vte_terminal_accessible_update_private_data_if_needed(ATK_OBJECT(text));
+	vte_terminal_accessible_update_private_data_if_needed(ATK_OBJECT(text),
+							      NULL);
 	/* FIXME? */
 	return FALSE;
 }
@@ -776,7 +1075,8 @@ vte_terminal_accessible_remove_selection(AtkText *text,
 					 gint selection_number)
 {
 	g_return_val_if_fail(VTE_IS_TERMINAL_ACCESSIBLE(text), FALSE);
-	vte_terminal_accessible_update_private_data_if_needed(ATK_OBJECT(text));
+	vte_terminal_accessible_update_private_data_if_needed(ATK_OBJECT(text),
+							      NULL);
 	/* FIXME? */
 	return FALSE;
 }
@@ -786,7 +1086,8 @@ vte_terminal_accessible_set_selection(AtkText *text, gint selection_number,
 				      gint start_offset, gint end_offset)
 {
 	g_return_val_if_fail(VTE_IS_TERMINAL_ACCESSIBLE(text), FALSE);
-	vte_terminal_accessible_update_private_data_if_needed(ATK_OBJECT(text));
+	vte_terminal_accessible_update_private_data_if_needed(ATK_OBJECT(text),
+							      NULL);
 	/* FIXME? */
 	return FALSE;
 }
@@ -795,7 +1096,8 @@ static gboolean
 vte_terminal_accessible_set_caret_offset(AtkText *text, gint offset)
 {
 	g_return_val_if_fail(VTE_IS_TERMINAL_ACCESSIBLE(text), FALSE);
-	vte_terminal_accessible_update_private_data_if_needed(ATK_OBJECT(text));
+	vte_terminal_accessible_update_private_data_if_needed(ATK_OBJECT(text),
+							      NULL);
 	/* Whoa, very not allowed. */
 	return FALSE;
 }
